/***************************************************************************
*   Copyright (C) 2004-2005 by Thomas Fischer                             *
*   fischer@unix-ag.uni-kl.de                                             *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
#include <bibtexencoder.h>
#include <bibtexcomment.h>

BibTeXComment::BibTeXComment( const QString &comment, QObject *parent, const char *name )
        : BibTeXElement( parent, name ), m_comment( BibTeXEncoder::currentBibTeXEncoder() ->decode( comment ) )
{
    //TODO
}

BibTeXComment::BibTeXComment( BibTeXComment *other )
{
    m_comment = other->m_comment;
}

BibTeXComment::~BibTeXComment()
{
    //TODO
}

void BibTeXComment::setComment( const QString &comment )
{
    m_comment = BibTeXEncoder::currentBibTeXEncoder() ->decode( comment );
}

QString BibTeXComment::comment()
{
    return m_comment;
}

QString BibTeXComment::toString( BibTeXFile::Format format, BibTeXFile::Encoding encoding )
{

    switch ( format )
    {
    case BibTeXFile::formatBibTeX:
        {
            QString result = QString::null;
            switch ( encoding )
            {
            case BibTeXFile::encImplicit :
            case BibTeXFile::encLaTeX :
                result = BibTeXEncoder::currentBibTeXEncoder() ->encode( m_comment );
            default:
                result = m_comment;
                break;
            }

            if ( result[ 0 ].latin1() == '%' )
            {
                QStringList lines = QStringList::split( '\n', result );
                result = QString( "" );
                for ( QStringList::Iterator it = lines.begin(); it != lines.end(); ++it )
                {
                    if ( ( *it ) [ 0 ] != '%' )
                        result.append( '%' );
                    result.append( *it ).append( '\n' );
                }
            }
            else
            {
                result = QString( "@comment {" ).append( result ).append( "}" );
            }

            return result;
        }
        break;
    case BibTeXFile::formatXML:
        return QString( " <comment>" ).append( m_comment ).append( "</comment>" );
        break;
    default:
        break;
    }

    return QString::null;
}

bool BibTeXComment::containsPattern( const QString& pattern, bool caseSensitive )
{
    return m_comment.find( pattern, 0, caseSensitive ) > 0;
}

#include "bibtexcomment.moc"
