/***************************************************************************
*   Copyright (C) 2004-2005 by Thomas Fischer                             *
*   fischer@unix-ag.uni-kl.de                                             *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
#ifndef BIBTEXENTRY_H
#define BIBTEXENTRY_H

#include <qstringlist.h>

#include <bibtexelement.h>
#include <bibtexentryfield.h>

class QStringList;

class BibTeXEntry : public BibTeXElement
{
    Q_OBJECT
public:
    enum EntryType {etArticle, etBook, etBooklet, etCollection, etElectronic, etInBook, etInCollection, etInProceedings, etManual, etMastersThesis, etMisc, etPhDThesis, etProceedings, etTechReport, etUnpublished, etUnknown};

    BibTeXEntry( const EntryType elementType, const QString &id, QObject *parent = 0, const char *name = 0 );
    BibTeXEntry( BibTeXEntry *other );
    ~BibTeXEntry();

    void setEntryType( const EntryType elementType );
    EntryType entryType();
    void setId( const QString& id );
    QString id();

    QString toString( BibTeXFile::Format format = BibTeXFile::formatBibTeX, BibTeXFile::Encoding encoding = BibTeXFile::encImplicit );
    bool containsPattern( const QString& pattern, bool caseSensitive = false );

    bool addField( BibTeXEntryField *field );
    BibTeXEntryField* getField( const BibTeXEntryField::FieldType fieldType );
    BibTeXEntryField* getField( const QString& key );
    bool deleteField( const BibTeXEntryField::FieldType fieldType );
    bool deleteField( const QString &key );

    QValueList<BibTeXEntryField*>::iterator begin();
    QValueList<BibTeXEntryField*>::iterator end();
    int getFieldCount();

    static QString entryTypeToString( const EntryType entryType );
    static EntryType entryTypeFromString( const QString &entryTypeString );
    static BibTeXEntryField::FieldRequireStatus getRequireStatus( BibTeXEntry::EntryType elementType, BibTeXEntryField::FieldType fieldType );


private:
    EntryType m_entryType;
    QString m_id;
    QValueList<BibTeXEntryField*> m_fields;
};

#endif
