/***************************************************************************
*   Copyright (C) 2004-2005 by Thomas Fischer                             *
*   fischer@unix-ag.uni-kl.de                                             *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
#ifndef BIBTEXFILE_H
#define BIBTEXFILE_H

#include <qobject.h>
#include <qvaluelist.h>

class QDir;
class QString;
class QChar;
class QStringList;
class QWaitCondition;
class QProcess;
class QIODevice;

class BibTeXElement;
class BibTeXEntry;
class BibTeXString;
class BibTeXFileParser;
class BibTeXFileExporter;

static const QString Months[] =
    {
        QString( "January" ), QString( "February" ), QString( "March" ), QString( "April" ), QString( "May" ), QString( "June" ), QString( "July" ), QString( "August" ), QString( "September" ), QString( "October" ), QString( "November" ), QString( "December" )
    };

static const QString MonthsTriple[] =
    {
        QString( "jan" ), QString( "feb" ), QString( "mar" ), QString( "apr" ), QString( "may" ), QString( "jun" ), QString( "jul" ), QString( "aug" ), QString( "sep" ), QString( "oct" ), QString( "nov" ), QString( "dec" )
    };

class BibTeXFile : public QObject
{
    Q_OBJECT

    friend class BibTeXFileExporter;
    friend class BibTeXFileParser;

public:
    enum Format { formatUndefined, formatBibTeX, formatXML, formatHTML, formatPDF, formatPS };
    enum Encoding {encImplicit = 0, encLaTeX = 1, encUTF8 = 2};
    enum StringProtection { spNone, spParanthesis, spQuote, spBoth };

    BibTeXFile( QObject *parent = 0, const char *name = 0 );
    ~BibTeXFile();

    // load a local BibTeX file
    bool load( const QString &filename );
    bool load( QIODevice *iodevice );
    bool load( QTextStream &stream );
    bool save( const QString &filename, BibTeXFile::Format format = BibTeXFile::formatBibTeX, BibTeXFile::Encoding encoding = BibTeXFile::encImplicit );
    bool save( QIODevice *iodevice, BibTeXFile::Format format = BibTeXFile::formatBibTeX, BibTeXFile::Encoding encoding = BibTeXFile::encImplicit );

    unsigned int count ();
    BibTeXElement* at( const unsigned int index );
    void appendElement( BibTeXElement *element );
    void deleteElement( BibTeXElement *element );

    void insertText( QString &text );

signals:
    void parseError( int errorId );
    void progress( int current, int total );

public slots:
    void cancel();

protected:
    typedef QValueList<BibTeXElement*> ElementList;
    ElementList elements;

private:
    void clearElements();

    BibTeXFileParser *parser;
    BibTeXFileExporter *exporter;

private slots:
    void slotProgress( int current, int total );

};




#endif
