/***************************************************************************
*   Copyright (C) 2004-2005 by Thomas Fischer                             *
*   fischer@unix-ag.uni-kl.de                                             *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
#include <qlabel.h>
#include <qmultilineedit.h>
#include <qlayout.h>
#include <kdialogbase.h>
#include <klocale.h>
#include <kmessagebox.h>

#include <bibtexcomment.h>

#include "kbibtexcommentwidget.h"

KBibTeXCommentWidget::KBibTeXCommentWidget( BibTeXComment *comment, bool isReadWrite, QWidget *parent, const char *name )
        : QWidget( parent, name ), m_comment( comment )
{
    setupGUI();
    getCommentData();
    m_multiLineEdit->setReadOnly( !isReadWrite );
}


KBibTeXCommentWidget::~KBibTeXCommentWidget()
{}

void KBibTeXCommentWidget::setupGUI()
{
    setMinimumWidth( 384 );
    QVBoxLayout * layout = new QVBoxLayout( this, 0, KDialog::spacingHint() );
    QLabel *label = new QLabel( i18n( "Co&mment:" ), this );
    layout->addWidget( label );
    label->show();
    m_multiLineEdit = new QMultiLineEdit( this );
    layout->addWidget( m_multiLineEdit );
    m_multiLineEdit->show();
    label->setBuddy( m_multiLineEdit );
}

void KBibTeXCommentWidget::getCommentData()
{
    m_multiLineEdit->setText( m_comment->comment() );
}

bool KBibTeXCommentWidget::setCommentData()
{
    bool result = true;
    QStringList warningElements;

    if ( m_multiLineEdit->text().isEmpty() )
        warningElements.append( i18n( "The comment text is empty" ) );
    m_comment->setComment( m_multiLineEdit->text() );

    if ( warningElements.count() > 0 )
        result = KMessageBox::warningContinueCancelList( this, i18n( "The comment is correct, as the following problems have been found. Would you to continue anyway?" ), warningElements, i18n( "Warnings for BibTeX element" ) ) == KMessageBox::Continue;

    return result;
}

QDialog::DialogCode KBibTeXCommentWidget::execute( BibTeXComment *comment, const QString& caption, bool isReadWrite, QWidget *parent, const char *name )
{
    KDialogBase * dlg = new KDialogBase( parent, name, true, caption, KDialogBase::Ok | KDialogBase::Cancel );
    KBibTeXCommentWidget* ui = new KBibTeXCommentWidget( comment, isReadWrite, dlg, "kbibtexcommentwidget" );

    dlg->setMainWidget( ui );
    QDialog::DialogCode result = ( QDialog::DialogCode ) dlg->exec();
    if ( isReadWrite && result == QDialog::Accepted )
        ui->setCommentData();

    delete( ui );
    delete( dlg );

    return result;
}

#include "kbibtexcommentwidget.moc"

