/***************************************************************************
*   Copyright (C) 2004-2005 by Thomas Fischer                             *
*   fischer@unix-ag.uni-kl.de                                             *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
#include <qlayout.h>
#include <qlistbox.h>
#include <qlineedit.h>
#include <qstringlist.h>
#include <qpushbutton.h>
#include <qstring.h>
#include <qlabel.h>

#include "kbibtexeditlistbox.h"

KBibTeXEditListBox::KBibTeXEditListBox( const QString& textLabel, QWidget *parent, const char *name ) : QWidget( parent, name )
{
    QGridLayout * layout = new QGridLayout( this, 7, 2, 0, 4 );
    layout->setRowStretch( layout->numRows() - 1, 1 );

    m_label = new QLabel( textLabel, this, "qeditlistbox_m_label" );
    layout->addMultiCellWidget( m_label, 0, 0, 0, 1 );
    m_lineEditInput = new QLineEdit( this, "qeditlistbox_lineeditinput" );
    layout->addWidget( m_lineEditInput, 1, 0 );
    m_label->setBuddy( m_lineEditInput );

    m_pushButtonAdd = new QPushButton( this, "qeditlistbox_pushbuttonadd" );
    layout->addWidget( m_pushButtonAdd, 1, 1 );
    m_pushButtonAdd->setEnabled( false );

    m_listBox = new QListBox( this, "qeditlistbox_listbox" );
    layout->addMultiCellWidget( m_listBox, 2, 6, 0, 0 );

    m_pushButtonModify = new QPushButton( this, "qeditlistbox_pushbuttonmodify" );
    layout->addWidget( m_pushButtonModify, 2, 1 );
    m_pushButtonDelete = new QPushButton( this, "qeditlistbox_pushbuttondelete" );
    layout->addWidget( m_pushButtonDelete, 3, 1 );
    m_pushButtonUp = new QPushButton( this, "qeditlistbox_pushbuttonnewup" );
    layout->addWidget( m_pushButtonUp, 4, 1 );
    m_pushButtonDown = new QPushButton( this, "qeditlistbox_pushbuttonnewdown" );
    layout->addWidget( m_pushButtonDown, 5, 1 );

    setFocusProxy( m_lineEditInput );

    updateUI();

    connect( m_lineEditInput, SIGNAL( textChanged( const QString& ) ), this, SLOT( slotLineEditTextChanged( ) ) );
    connect( m_pushButtonAdd, SIGNAL( clicked() ), this, SLOT( slotPushButtonAddClicked() ) );
    connect( m_pushButtonModify, SIGNAL( clicked() ), this, SLOT( slotPushButtonModifyClicked() ) );
    connect( m_pushButtonDelete, SIGNAL( clicked() ), this, SLOT( slotPushButtonDeleteClicked() ) );
    connect( m_pushButtonUp, SIGNAL( clicked() ), this, SLOT( slotPushButtonUpClicked() ) );
    connect( m_pushButtonDown, SIGNAL( clicked() ), this, SLOT( slotPushButtonDownClicked() ) );
    connect( m_listBox, SIGNAL( highlighted( int ) ), this, SLOT( slotListBoxHighlighted( ) ) );
}

QStringList KBibTeXEditListBox::lines() const
{
    QListBoxItem * item = m_listBox->firstItem();
    QStringList items;
    while ( item )
    {
        items << item->text();
        item = item->next();
    }

    return items;
}

void KBibTeXEditListBox::setLines( const QStringList &lines )
{
    m_listBox->clear();
    m_listBox->insertStringList( lines );
}

void KBibTeXEditListBox::setButtonText( buttons button, const QString& text )
{
    switch ( button )
    {
    case Add: m_pushButtonAdd->setText( text ); break;
    case Modify: m_pushButtonModify->setText( text ); break;
    case Delete: m_pushButtonDelete->setText( text ); break;
    case Up: m_pushButtonUp->setText( text ); break;
    case Down: m_pushButtonDown->setText( text ); break;
    }
}

void KBibTeXEditListBox::slotLineEditTextChanged( )
{
    updateUI();
}

void KBibTeXEditListBox::slotPushButtonAddClicked()
{
    int index = m_listBox->currentItem();
    m_listBox->insertItem( m_lineEditInput->text() );
    m_listBox->setSelected( index, FALSE );
    m_listBox->setCurrentItem( -1 );
    m_lineEditInput->setText( "" );
}

void KBibTeXEditListBox::slotPushButtonModifyClicked()
{
    QString text = m_lineEditInput->text();
    int index = m_listBox->currentItem();
    m_listBox->removeItem( index );
    m_listBox->insertItem( text, index );
    m_listBox->setCurrentItem( index );
}

void KBibTeXEditListBox::slotPushButtonDeleteClicked()
{
    int index = m_listBox->currentItem();
    m_listBox->removeItem( index );
    //m_listBox->setCurrentItem( ( index - 1 ) < 0 ? 0 : index - 1 );
    //m_listBox->setSelected( index - 1, TRUE );
}

void KBibTeXEditListBox::slotPushButtonUpClicked()
{
    QString text = m_listBox->currentText();
    int index = m_listBox->currentItem();
    m_listBox->removeItem( index );
    m_listBox->insertItem( text, index - 1 );
    m_listBox->setCurrentItem( index - 1 );
}

void KBibTeXEditListBox::slotPushButtonDownClicked()
{
    QString text = m_listBox->currentText();
    int index = m_listBox->currentItem();
    m_listBox->removeItem( index );
    m_listBox->insertItem( text, index + 1 );
    m_listBox->setCurrentItem( index + 1 );
}

void KBibTeXEditListBox::slotListBoxHighlighted( )
{
    if ( m_listBox->isSelected( m_listBox->currentItem() ) )
        m_lineEditInput->setText( m_listBox->currentText() );
    else
        m_lineEditInput->setText( "" );
    updateUI();
}

void KBibTeXEditListBox::updateUI()
{
    int index = m_listBox->currentItem();
    bool curIsSel = m_listBox->isSelected( m_listBox->currentItem() );

    m_pushButtonAdd->setEnabled( !m_lineEditInput->text().isEmpty() );
    m_pushButtonModify->setEnabled( !m_lineEditInput->text().isEmpty() && curIsSel );
    m_pushButtonDelete->setEnabled( curIsSel && index > -1 );
    m_pushButtonUp->setEnabled( curIsSel && index > 0 );
    m_pushButtonDown->setEnabled( curIsSel && index > -1 && index < ( int ) ( m_listBox->count() - 1 ) );
}

#include "kbibtexeditlistbox.moc"
