/***************************************************************************
*   Copyright (C) 2004-2005 by Thomas Fischer                             *
*   fischer@unix-ag.uni-kl.de                                             *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

#include <qwidget.h>
#include <qcheckbox.h>
#include <qcombobox.h>
#include <qfont.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qlineedit.h>
#include <qlistview.h>
#include <qmessagebox.h>
#include <qobjectlist.h>
#include <qpushbutton.h>
#include <qspinbox.h>
#include <qsplitter.h>
#include <qstringlist.h>
#include <qtabwidget.h>
#include <qtextedit.h>
#include <qtextedit.h>
#include <qvbox.h>
#include <kdialogbase.h>
#include <kiconloader.h>
#include <klocale.h>
#include <kmessagebox.h>
#include <kglobalsettings.h>

#include <settings.h>
#include <bibtexfile.h>
#include <bibtexentry.h>
#include <keditlistbox.h>
#include <kbibtexselectstring.h>
#include <kbibtexentrywidgettab.h>
#include "kbibtexentrywidget.h"

KBibTeXEntryWidget::KBibTeXEntryWidget( BibTeXEntry *entry, BibTeXFile *bibtexfile, bool isReadWrite, QWidget *parent, const char *name )
        : QWidget( parent, name ), m_entry( entry ), m_bibtexfile( bibtexfile ), m_enableAllFields( false ), m_isReadWrite( isReadWrite )
{
    setupGUI();
    getEntryData( m_entry );
}

KBibTeXEntryWidget::~KBibTeXEntryWidget()
{}

BibTeXEntry::EntryType KBibTeXEntryWidget::currentEntryType()
{
    return ( BibTeXEntry::EntryType ) ( ( int ) BibTeXEntry::etArticle + m_comboBoxEntryType->currentItem() );
}

void KBibTeXEntryWidget::setupGUI()
{
    QLabel * label = NULL;
    QSpacerItem *spacer = NULL;
    QHBoxLayout *hBoxLayout = NULL;

    QVBoxLayout *mainlayout = new QVBoxLayout( this, 1, 6, "mainlayout" );

    hBoxLayout = new QHBoxLayout( 0, 0, 6, "layout" );
    label = new QLabel( i18n( "E&ntry Type:" ), this );
    hBoxLayout->addWidget( label );
    m_comboBoxEntryType = new QComboBox( FALSE, this, "m_comboBoxEntryType" );
    label->setBuddy( m_comboBoxEntryType );
    m_comboBoxEntryType->setSizePolicy( QSizePolicy::MinimumExpanding, QSizePolicy::Fixed );
    m_comboBoxEntryType->setEnabled( m_isReadWrite );
    m_comboBoxEntryType->installEventFilter( this );
    hBoxLayout->addWidget( m_comboBoxEntryType );

    spacer = new QSpacerItem( 24, 20, QSizePolicy::Fixed, QSizePolicy::Minimum );
    hBoxLayout->addItem( spacer );

    label = new QLabel( i18n( "&Identifier" ), this );
    hBoxLayout->addWidget( label );
    m_lineEditID = new QLineEdit( this, "m_lineEditID" );
    label->setBuddy( m_lineEditID );
    m_lineEditID->setEnabled( m_isReadWrite );
    m_lineEditID->setSizePolicy( QSizePolicy::MinimumExpanding, QSizePolicy::Fixed );
    m_lineEditID->installEventFilter( this );
    hBoxLayout->addWidget( m_lineEditID );

    mainlayout->addLayout( hBoxLayout );

    m_tabWidgetMain = new QTabWidget( this, "m_tabWidgetMain" );

    // ================================================================
    // Tab "Title"
    // ----------------------------------------------------------------
    m_tabTitle = new KBibTeXEntryWidgetTitle( this, m_tabWidgetMain, "tab_title" );
    m_tabWidgetMain->insertTab( m_tabTitle, i18n( "Title" ) );

    // ================================================================
    // Tab "Author"
    // ----------------------------------------------------------------
    m_tabAuthor = new KBibTeXEntryWidgetAuthor( this, m_tabWidgetMain, "tab_author" );
    m_tabWidgetMain->insertTab( m_tabAuthor, i18n( "Author" ) );

    // ================================================================
    // Tab "Publication"
    // ----------------------------------------------------------------
    m_tabPublication = new KBibTeXEntryWidgetPublication( this, m_tabWidgetMain, "tab_publication" );
    m_tabWidgetMain->insertTab( m_tabPublication, i18n( "Publication" ) );

    // ================================================================
    // Tab "Misc"
    // ----------------------------------------------------------------
    m_tabMisc = new KBibTeXEntryWidgetMisc( this, m_tabWidgetMain, "tab_misc" );
    m_tabWidgetMain->insertTab( m_tabMisc, i18n( "Misc" ) );

    // ================================================================
    // Tab "User Fields"
    // ----------------------------------------------------------------
    m_tabUserFields = new KBibTeXEntryWidgetUserFields( this, m_tabWidgetMain, "tab_userfields" );
    m_tabWidgetMain->insertTab( m_tabUserFields, i18n( "User Fields" ) );

    // ================================================================
    // Tab "Source"
    // ----------------------------------------------------------------
    m_tabSource = new KBibTeXEntryWidgetSource( this, m_tabWidgetMain, "tab_source" );
    m_tabWidgetMain->insertTab( m_tabSource, i18n( "Source" ) );

    // ================================================================
    // other stuff
    // ----------------------------------------------------------------
    mainlayout->addWidget( m_tabWidgetMain );

    // ================================================================
    // List with warnings
    // ----------------------------------------------------------------
    m_listViewWarnings = new QListView( this );

    m_listViewWarnings->addColumn( i18n( "Level" ) );
    m_listViewWarnings->addColumn( i18n( "Message" ) );
    m_listViewWarnings->setAllColumnsShowFocus( true );
    mainlayout->addWidget( m_listViewWarnings );

    resize( QSize( 768, 480 ).expandedTo( minimumSizeHint() ) );
    clearWState( WState_Polished );

    setupEntryTypes();

    connect( m_comboBoxEntryType, SIGNAL( activated( int ) ), this, SLOT( slotEntryTypeChanged( int ) ) );
    connect( m_tabWidgetMain, SIGNAL( currentChanged( QWidget* ) ), this, SLOT( slotCurrentTabChanged( QWidget* ) ) );
    connect( m_listViewWarnings, SIGNAL( doubleClicked( QListViewItem*, const QPoint&, int ) ), this, SLOT( warningsExecute( QListViewItem* ) ) );

    updateWarnings( );
}

void KBibTeXEntryWidget::setupEntryTypes()
{
    int i;
    BibTeXEntry::EntryType entryType;

    i = ( int ) BibTeXEntry::etArticle;
    entryType = ( BibTeXEntry::EntryType ) i;
    while ( entryType != BibTeXEntry::etUnknown )
    {
        m_comboBoxEntryType->insertItem( BibTeXEntry::entryTypeToString( entryType ) );
        entryType = ( BibTeXEntry::EntryType ) ++i;
    }
}

void KBibTeXEntryWidget::getEntryData( BibTeXEntry *entry , bool fromSource )
{
    m_comboBoxEntryType->setCurrentItem( entry->entryType() - ( int ) BibTeXEntry::etArticle );
    m_lineEditID->setText( entry->id() );

    m_tabTitle ->getEntryData( entry );
    m_tabAuthor ->getEntryData( entry );
    m_tabPublication->getEntryData( entry );
    m_tabMisc ->getEntryData( entry );
    m_tabUserFields->getEntryData( entry );
    if ( !fromSource )
        m_tabSource->getEntryData( entry );

    changeEntryType( entry->entryType() );
}

bool KBibTeXEntryWidget::setEntryData( BibTeXEntry *entry )
{
    bool result = true;
    QString text;

    entry->setEntryType( currentEntryType() );
    text = m_lineEditID->text();
    entry->setId( text );

    m_tabTitle->setEntryData( entry );
    m_tabAuthor->setEntryData( entry );
    m_tabPublication->setEntryData( entry );
    m_tabMisc->setEntryData( entry );
    m_tabUserFields->setEntryData( entry );

    return result;
}

QDialog::DialogCode KBibTeXEntryWidget::execute( BibTeXEntry *entry, BibTeXFile *bibtexfile, const QString& caption, bool isReadWrite, QWidget *parent, const char *name )
{
    KDialogBase * dlg = new KDialogBase( parent, name, true, caption, KDialogBase::Ok | KDialogBase::Cancel );
    QVBox *vbox = new QVBox( dlg );
    KBibTeXEntryWidget* entryWidget = new KBibTeXEntryWidget( entry, bibtexfile, isReadWrite, vbox, "kbibtexentrywidget" );
    connect( dlg, SIGNAL( okClicked() ), entryWidget, SLOT( slotOk() ) );

    QCheckBox *cb = new QCheckBox( i18n( "Enable all &fields for editing" ), vbox );
    cb->setChecked( kbibtexsettings.enableallfields );
    entryWidget->slotEnableAllFields( kbibtexsettings.enableallfields );
    connect( cb, SIGNAL( toggled( bool ) ), entryWidget, SLOT( slotEnableAllFields( bool ) ) );

    dlg->setMainWidget( vbox );
    QDialog::DialogCode result = ( QDialog::DialogCode ) dlg->exec();

    delete( entryWidget );
    delete( dlg );

    return result;
}

void KBibTeXEntryWidget::slotOk()
{
    setEntryData( m_entry );
}

void KBibTeXEntryWidget::changeEntryType( BibTeXEntry::EntryType entryType )
{
    m_tabTitle ->changeEntryType( entryType, m_enableAllFields , m_isReadWrite );
    m_tabAuthor ->changeEntryType( entryType, m_enableAllFields , m_isReadWrite );
    m_tabPublication ->changeEntryType( entryType, m_enableAllFields, m_isReadWrite );
    m_tabMisc ->changeEntryType( entryType, m_enableAllFields, m_isReadWrite );
    m_tabUserFields ->changeEntryType( entryType, m_enableAllFields, m_isReadWrite );
    m_tabSource ->changeEntryType( entryType, m_enableAllFields, m_isReadWrite );

    updateWarnings( );
}

void KBibTeXEntryWidget::slotEntryTypeChanged( int index )
{
    changeEntryType( ( BibTeXEntry::EntryType ) ( ( int ) BibTeXEntry::etArticle + index ) );
}

void KBibTeXEntryWidget::slotEnableAllFields( bool enable )
{
    m_enableAllFields = enable;
    changeEntryType( ( BibTeXEntry::EntryType ) ( ( int ) BibTeXEntry::etArticle + m_comboBoxEntryType->currentItem() ) );
}

void KBibTeXEntryWidget::slotCurrentTabChanged( QWidget* newTab )
{
    if ( newTab == m_tabSource )
    {
        m_tabSource->setModified( FALSE );
        refreshSourceView();
    }
    else if ( m_tabSource->isModified() )
    {
        m_tabSource->setModified( FALSE );
        parseSourceView();
    }
}

void KBibTeXEntryWidget::refreshSourceView()
{
    BibTeXEntry * tmpEntry = new BibTeXEntry( m_entry );
    setEntryData( tmpEntry );
    m_tabSource->getEntryData( tmpEntry );
    delete tmpEntry;
}

void KBibTeXEntryWidget::parseSourceView()
{
    BibTeXEntry * entry = new BibTeXEntry( BibTeXEntry::etUnknown, QString::null );
    if ( m_tabSource->setEntryData( entry ) )
        getEntryData( entry, true );
    else
        KMessageBox::error( this, i18n( "Your modifications on the bibtex source code could not be interpreted." ), i18n( "Parsing error" ) );

    delete entry;
}

void KBibTeXEntryWidget::warningsExecute( QListViewItem* item )
{
    KBibTeXEntryWidgetWarningsItem * ewwi = dynamic_cast<KBibTeXEntryWidgetWarningsItem*>( item );
    if ( ewwi != NULL )
    {
        ewwi->widget() ->setFocus();
        // find and activate corresponding tab page
        QObject *parent = ewwi->widget();
        KBibTeXEntryWidgetTab *ewt = dynamic_cast<KBibTeXEntryWidgetTab*>( parent );
        while ( ewt == NULL && parent != NULL )
        {
            parent = parent->parent();
            ewt = dynamic_cast<KBibTeXEntryWidgetTab*>( parent );
        }
        m_tabWidgetMain->setCurrentPage( m_tabWidgetMain-> indexOf( ewt ) );
    }
}

void KBibTeXEntryWidget::updateWarnings( )
{
    //FIXME Delete m_listViewWarnings elements manually?
    m_listViewWarnings->clear();
    m_tabTitle ->updateWarnings( );
    m_tabAuthor ->updateWarnings( );
    m_tabPublication ->updateWarnings( );
    m_tabMisc ->updateWarnings( );
    m_tabSource ->updateWarnings( );

    if ( m_lineEditID->text().length() == 0 )
        new KBibTeXEntryWidgetWarningsItem( KBibTeXEntryWidgetWarningsItem::wlError, i18n( "An entry has to have an identifier" ), m_lineEditID, m_listViewWarnings, "error" );
}

bool KBibTeXEntryWidget::eventFilter( QObject* /* object*/, QEvent *event )
{
    if ( event->type() == QEvent::FocusOut )
        updateWarnings( );

    return FALSE;
}

KBibTeXEntryWidgetWarningsItem::KBibTeXEntryWidgetWarningsItem( WarningLevel level, const QString &message, QWidget *widget, QListView *parent, const char* name )
        : QListViewItem( parent, name ), m_level( level ), m_message( message ), m_widget( widget )
{
    updateItem();
}

KBibTeXEntryWidgetWarningsItem::~KBibTeXEntryWidgetWarningsItem()
{
    // TODO
}

void KBibTeXEntryWidgetWarningsItem::updateItem()
{
    KIconLoader * loader = KGlobal::instance() ->iconLoader();
    switch ( m_level )
    {
    case wlInformation:
        setPixmap( 0, loader->loadIcon( "messagebox_info", KIcon::NoGroup, KIcon::SizeSmall ) );
        setText( 0, i18n( "Information" ) );
        break;
    case wlWarning:
        setPixmap( 0, loader->loadIcon( "messagebox_warning", KIcon::NoGroup, KIcon::SizeSmall ) );
        setText( 0, i18n( "Warning" ) );
        break;
    case wlError:
        setPixmap( 0, loader->loadIcon( "messagebox_critical", KIcon::NoGroup, KIcon::SizeSmall ) );
        setText( 0, i18n( "Error" ) );
        break;
    default:
        setText( 0, i18n( "Unknown" ) );
    }
    setText( 1, m_message );
}

QWidget *KBibTeXEntryWidgetWarningsItem::widget()
{
    return m_widget;
}

#include "kbibtexentrywidget.moc"
