/***************************************************************************
*   Copyright (C) 2004-2005 by Thomas Fischer                             *
*   fischer@unix-ag.uni-kl.de                                             *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

#include <qlineedit.h>
#include <qlayout.h>
#include <qlabel.h>
#include <qtextedit.h>
#include <qspinbox.h>
#include <qcombobox.h>
#include <qstringlist.h>
#include <qobjectlist.h>
#include <qpushbutton.h>
#include <qlistview.h>
#include <qregexp.h>

#include <klocale.h>
#include <kdialog.h>
#include <kurl.h>
#include <kapplication.h>
#include <kglobalsettings.h>

#include <bibtexentry.h>
#include <kbibtexentrywidget.h>
#include <kbibtexeditlistbox.h>
#include <kbibtexfieldlistviewitem.h>

#include "kbibtexentrywidgettab.h"

static const QString MonthsI18N[] =
    {
        i18n( "January" ), i18n( "February" ), i18n( "March" ), i18n( "April" ), i18n( "May" ), i18n( "June" ), i18n( "July" ), i18n( "August" ), i18n( "September" ), i18n( "October" ), i18n( "November" ), i18n( "December" )
    };
static const QString warningMsgMissingField = QString( i18n( "The field '%1' is required, but missing" ) );
static const QString warningMsgOptionalField = QString( i18n( "The field '%1' is optional and may be set" ) );

KBibTeXEntryWidgetTab::KBibTeXEntryWidgetTab( KBibTeXEntryWidget *entryWidget, QWidget *parent, const char *name ) : QWidget( parent, name ), m_entryWidget( entryWidget )
{
    // nothing
}

KBibTeXEntryWidgetTab::~KBibTeXEntryWidgetTab()
{
    // nothing
}

void KBibTeXEntryWidgetTab::setupGUI()
{
    // nothing
}

void KBibTeXEntryWidgetTab::getEntryData( BibTeXEntry * /* entry */ )
{
    // nothing
}

bool KBibTeXEntryWidgetTab::setEntryData( BibTeXEntry * /* entry */ )
{
    return false;
}

void KBibTeXEntryWidgetTab::changeEntryType( BibTeXEntry::EntryType /* entryType */, bool /* enableAll */, bool /* isReadWrite */ )
{
    // nothing
}

void KBibTeXEntryWidgetTab::updateWarnings()
{
    // nothing
}

bool KBibTeXEntryWidgetTab::setEntryDataSpinBox( BibTeXEntry *entry, QSpinBox *sbx, BibTeXEntryField::FieldType fieldType )
{
    if ( sbx->value() > sbx->minValue() )
    {
        QString text = QString::number( sbx->value() );
        BibTeXEntryField* field = entry->getField( fieldType );
        if ( field == NULL )
        {
            field = new BibTeXEntryField( fieldType, text );
            entry->addField( field );
        }
        else
            field->setValue( text );
        return true;
    }
    else
    {
        entry->deleteField( fieldType );
        return false;
    }
}

bool KBibTeXEntryWidgetTab::setEntryDataText( BibTeXEntry *entry, const QString& text, BibTeXEntryField::FieldType fieldType )
{
    if ( !text.isEmpty() )
    {
        BibTeXEntryField * field = entry->getField( fieldType );
        if ( field == NULL )
        {
            field = new BibTeXEntryField( fieldType, text );
            entry->addField( field );
        }
        else
            field->setValue( text );
    }
    else
        entry->deleteField( fieldType );

    //     return ( BibTeXEntry::getRequireStatus( ( BibTeXEntry::EntryType ) ( ( int ) BibTeXEntry::etArticle + m_comboBoxEntryType->currentItem() ), fieldType ) != BibTeXEntryField::frsRequired ) || !text.isEmpty();
    return true;
}

void KBibTeXEntryWidgetTab::setWidgetStatus( QWidget* widget, BibTeXEntryField::FieldRequireStatus requiredStatus, bool enableAll, bool isReadWrite )
{
    widget->setEnabled ( ( requiredStatus != BibTeXEntryField::frsIgnored ) || enableAll );

    if ( !isReadWrite )
    {
        QLineEdit * lineEdit = dynamic_cast<QLineEdit*>( widget );
        if ( lineEdit != NULL )
            lineEdit->setReadOnly( TRUE );
        else
            widget->setEnabled( FALSE );
    }
}

void KBibTeXEntryWidgetTab::addWarning( BibTeXEntryField::FieldType fieldType, const QString& label, bool valid, QWidget *widget )
{
    if ( BibTeXEntry::getRequireStatus( m_entryWidget->currentEntryType(), fieldType ) == BibTeXEntryField::frsRequired && !valid )
        new KBibTeXEntryWidgetWarningsItem( KBibTeXEntryWidgetWarningsItem::wlWarning, warningMsgMissingField.arg( label ), widget, m_entryWidget->m_listViewWarnings, "warning" );
    //     else if ( BibTeXEntry::getRequireStatus( m_entryWidget->currentEntryType(), fieldType ) == BibTeXEntryField::frsOptional && !valid )
    //         new KBibTeXEntryWidgetWarningsItem( KBibTeXEntryWidgetWarningsItem::wlInformation, warningMsgOptionalField.arg( label ), widget, m_entryWidget->m_listViewWarnings, "information" );
}

void KBibTeXEntryWidgetTab::installEventFilters( QWidget *parent )
{
    for ( QObjectListIterator it( *( parent-> children() ) ); it.current(); ++it )
    {
        QWidget *widget = dynamic_cast<QWidget*>( it.current() );
        if ( widget )
            widget->installEventFilter( m_entryWidget );
    }
}

// ============================================================================================

KBibTeXEntryWidgetTitle::KBibTeXEntryWidgetTitle( KBibTeXEntryWidget * entryWidget, QWidget * parent, const char * name )
        : KBibTeXEntryWidgetTab( entryWidget, parent, name )
{
    setupGUI();
}

KBibTeXEntryWidgetTitle::~KBibTeXEntryWidgetTitle()
{
    //TODO
}

void KBibTeXEntryWidgetTitle::setupGUI()
{
    QGridLayout * gridLayout = new QGridLayout( this, 1, 1, 11, KDialog::spacingHint(), "gridLayout" );

    QLabel *label = new QLabel( i18n( "&Title:" ), this );
    m_lineEditTitle = new QLineEdit( this, "m_lineEditTitle" );
    label->setBuddy( m_lineEditTitle );
    gridLayout->addWidget( label, 0, 0 );
    gridLayout->addWidget( m_lineEditTitle, 0, 1 );

    label = new QLabel( i18n( "&Book Title:" ), this );
    m_lineEditBookTitle = new QLineEdit( this, "m_lineEditBookTitle" );
    label->setBuddy( m_lineEditBookTitle );
    gridLayout->addWidget( label, 1, 0 );
    gridLayout->addWidget( m_lineEditBookTitle, 1, 1 );

    label = new QLabel( i18n( "&Series:" ), this );
    m_lineEditSeries = new QLineEdit( this, "m_lineEditSeries" );
    label->setBuddy( m_lineEditSeries );
    gridLayout->addWidget( label, 2, 0 );
    gridLayout->addWidget( m_lineEditSeries, 2, 1 );

    QSpacerItem *spacer = new QSpacerItem( 20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding );
    gridLayout->addItem( spacer, 3, 1 );

    installEventFilters( this );
}

void KBibTeXEntryWidgetTitle::getEntryData( BibTeXEntry * entry )
{
    BibTeXEntryField * field = entry->getField( BibTeXEntryField::ftTitle );
    if ( field )
        m_lineEditTitle->setText( field->value() );
    field = entry->getField( BibTeXEntryField::ftBookTitle );
    if ( field )
        m_lineEditBookTitle->setText( field->value() );
    field = entry->getField( BibTeXEntryField::ftSeries );
    if ( field )
        m_lineEditSeries->setText( field->value() );
}

bool KBibTeXEntryWidgetTitle::setEntryData( BibTeXEntry * entry )
{
    bool result = true;

    result &= setEntryDataText( entry, m_lineEditTitle->text(), BibTeXEntryField::ftTitle );
    result &= setEntryDataText( entry, m_lineEditBookTitle->text(), BibTeXEntryField::ftBookTitle );
    result &= setEntryDataText( entry, m_lineEditSeries->text(), BibTeXEntryField::ftSeries );

    return result;
}

void KBibTeXEntryWidgetTitle::changeEntryType( BibTeXEntry::EntryType entryType, bool enableAll, bool isReadWrite )
{
    setWidgetStatus( m_lineEditTitle, BibTeXEntry::getRequireStatus( entryType, BibTeXEntryField::ftTitle ) , enableAll, isReadWrite );
    setWidgetStatus( m_lineEditBookTitle, BibTeXEntry::getRequireStatus( entryType, BibTeXEntryField::ftBookTitle ) , enableAll, isReadWrite );
    setWidgetStatus( m_lineEditSeries, BibTeXEntry::getRequireStatus( entryType, BibTeXEntryField::ftSeries ) , enableAll, isReadWrite );
}

void KBibTeXEntryWidgetTitle::updateWarnings()
{
    addWarning( BibTeXEntryField::ftTitle, i18n( "Title" ), !m_lineEditTitle->text().isEmpty(), m_lineEditTitle );
    addWarning( BibTeXEntryField::ftBookTitle, i18n( "Book Title" ), !m_lineEditBookTitle->text().isEmpty(), m_lineEditBookTitle );
    addWarning( BibTeXEntryField::ftSeries, i18n( "Series" ), !m_lineEditSeries->text().isEmpty(), m_lineEditSeries );
}

// ============================================================================================

KBibTeXEntryWidgetAuthor::KBibTeXEntryWidgetAuthor( KBibTeXEntryWidget * entryWidget, QWidget * parent, const char * name )
        : KBibTeXEntryWidgetTab( entryWidget, parent, name )
{
    setupGUI();
}

KBibTeXEntryWidgetAuthor::~KBibTeXEntryWidgetAuthor()
{
    //TODO
}

void KBibTeXEntryWidgetAuthor::setupGUI()
{
    QHBoxLayout * hBoxLayout = new QHBoxLayout( this, KDialog::marginHint(), KDialog::spacingHint(), "hBoxLayout" );
    m_editListBoxAuthor = new KBibTeXEditListBox( i18n( "&Author:" ), this, "elbAuthor" );
    hBoxLayout->addWidget( m_editListBoxAuthor );

    QSpacerItem *spacer = new QSpacerItem( 24, 20, QSizePolicy::Fixed, QSizePolicy::Minimum );
    hBoxLayout->addItem( spacer );

    m_editListBoxEditor = new KBibTeXEditListBox( i18n( "&Editor:" ), this, "elbEditor" );
    hBoxLayout->addWidget( m_editListBoxEditor );

    m_editListBoxAuthor->setButtonText( KBibTeXEditListBox::Add, i18n( "Add Au&thor" ) );
    m_editListBoxAuthor->setButtonText( KBibTeXEditListBox::Modify, i18n( "Modify" ) );
    m_editListBoxAuthor->setButtonText( KBibTeXEditListBox::Delete, i18n( "Remove" ) );
    m_editListBoxAuthor->setButtonText( KBibTeXEditListBox::Up, i18n( "&Up" ) );
    m_editListBoxAuthor->setButtonText( KBibTeXEditListBox::Down, i18n( "Do&wn" ) );
    m_editListBoxEditor->setButtonText( KBibTeXEditListBox::Add, i18n( "Add Edito&r" ) );
    m_editListBoxEditor->setButtonText( KBibTeXEditListBox::Modify, i18n( "Modify" ) );
    m_editListBoxEditor->setButtonText( KBibTeXEditListBox::Delete, i18n( "Remove" ) );
    m_editListBoxEditor->setButtonText( KBibTeXEditListBox::Up, i18n( "U&p" ) );
    m_editListBoxEditor->setButtonText( KBibTeXEditListBox::Down, i18n( "Dow&n" ) );

    installEventFilters( m_editListBoxAuthor );
    installEventFilters( m_editListBoxEditor );
}

void KBibTeXEntryWidgetAuthor::getEntryData( BibTeXEntry * entry )
{
    BibTeXEntryField * field = entry->getField( BibTeXEntryField::ftAuthor );
    if ( field )
        m_editListBoxAuthor->setLines( QStringList::split( QRegExp( "\\s+(,|and|&)\\s+", false ), field->value() ) );
    field = entry->getField( BibTeXEntryField::ftEditor );
    if ( field )
        m_editListBoxEditor->setLines( QStringList::split( QRegExp( "\\s+(,|and|&)\\s+", false ), field->value() ) );
}

bool KBibTeXEntryWidgetAuthor::setEntryData( BibTeXEntry * entry )
{
    bool result = true;
    result &= setEntryDataText( entry, m_editListBoxAuthor->lines().join( " and " ), BibTeXEntryField::ftAuthor );
    result &= setEntryDataText( entry, m_editListBoxEditor->lines().join( " and " ), BibTeXEntryField::ftEditor );
    return result;
}

void KBibTeXEntryWidgetAuthor::changeEntryType( BibTeXEntry::EntryType entryType, bool enableAll, bool isReadWrite )
{
    setWidgetStatus( m_editListBoxAuthor, BibTeXEntry::getRequireStatus( entryType, BibTeXEntryField::ftAuthor ) , enableAll, isReadWrite );
    setWidgetStatus( m_editListBoxEditor, BibTeXEntry::getRequireStatus( entryType, BibTeXEntryField::ftEditor ), enableAll, isReadWrite );
}

void KBibTeXEntryWidgetAuthor::updateWarnings()
{
    // FIXME: This type of checking should be done in BibTeXEntry or BibTeXEntryField
    if ( m_entryWidget->currentEntryType() == BibTeXEntry::etBook || m_entryWidget->currentEntryType() == BibTeXEntry::etInBook )
    {
        if ( m_editListBoxAuthor->lines().size() == 0 && m_editListBoxEditor->lines().size() == 0 )
            new KBibTeXEntryWidgetWarningsItem( KBibTeXEntryWidgetWarningsItem::wlWarning, i18n( "The fields 'Author' or 'Editor' are required, but missing" ), m_editListBoxAuthor, m_entryWidget->m_listViewWarnings, "warning" );
    }
    else
    {
        addWarning( BibTeXEntryField::ftAuthor, i18n( "Author" ), m_editListBoxAuthor->lines().size() > 0, m_editListBoxAuthor );
        addWarning( BibTeXEntryField::ftEditor, i18n( "Editor" ), m_editListBoxEditor->lines().size() > 0, m_editListBoxEditor );
    }

    // FIXME: Is this code really neccessary???
    if ( m_entryWidget->currentEntryType() == BibTeXEntry::etArticle && m_editListBoxEditor->lines().size() > 0 && m_editListBoxAuthor->lines().size() > 0 )
        new KBibTeXEntryWidgetWarningsItem( KBibTeXEntryWidgetWarningsItem::wlError, i18n( "You cannot use both 'Author' and 'Editor'" ), m_editListBoxEditor, m_entryWidget->m_listViewWarnings, "error" );

}

// ============================================================================================

KBibTeXEntryWidgetPublication::KBibTeXEntryWidgetPublication( KBibTeXEntryWidget * entryWidget, QWidget * parent, const char * name )
        : KBibTeXEntryWidgetTab( entryWidget, parent, name )
{
    setupGUI();
}

KBibTeXEntryWidgetPublication::~KBibTeXEntryWidgetPublication()
{
    //TODO
}

void KBibTeXEntryWidgetPublication::setupGUI()
{
    QGridLayout * gridLayout = new QGridLayout( this, 8, 14, KDialog::marginHint(), KDialog::spacingHint(), "gridLayout" );

    m_lineEditHowPublished = new QLineEdit( this, "m_lineEditHowPublished" );
    m_lineEditHowPublished->setSizePolicy( QSizePolicy::MinimumExpanding, QSizePolicy::Fixed );
    gridLayout->addMultiCellWidget( m_lineEditHowPublished, 0, 0, 1, 5 );
    QLabel *label = new QLabel( i18n( "Ho&w Published:" ), this, "textLabelHowPublished" );
    label->setBuddy( m_lineEditHowPublished );
    gridLayout->addWidget( label, 0, 0 );

    m_lineEditPublisher = new QLineEdit( this, "m_lineEditPublisher" );
    m_lineEditPublisher->setSizePolicy( QSizePolicy::MinimumExpanding, QSizePolicy::Fixed );
    gridLayout->addMultiCellWidget( m_lineEditPublisher, 0, 0, 8, 12 );
    label = new QLabel( i18n( "&Publisher:" ), this, "textLabelPublisher" );
    label->setBuddy( m_lineEditPublisher );
    gridLayout->addWidget( label, 0, 7 );

    m_lineEditOrganization = new QLineEdit( this, "m_lineEditOrganization" );
    m_lineEditOrganization->setSizePolicy( QSizePolicy::MinimumExpanding, QSizePolicy::Fixed );
    gridLayout->addMultiCellWidget( m_lineEditOrganization, 1, 1, 1, 5 );
    label = new QLabel( i18n( "Org&anization" ), this, "textLabelOrganization" );
    label->setBuddy( m_lineEditOrganization );
    gridLayout->addWidget( label, 1, 0 );

    m_lineEditInstitution = new QLineEdit( this, "m_lineEditInstitution" );
    m_lineEditInstitution->setSizePolicy( QSizePolicy::MinimumExpanding, QSizePolicy::Fixed );
    gridLayout->addMultiCellWidget( m_lineEditInstitution, 1, 1, 8, 12 );
    label = new QLabel( i18n( "Instit&ution:" ), this, "textLabelInstitution" );
    label->setBuddy( m_lineEditInstitution );
    gridLayout->addWidget( label, 1, 7 );

    m_lineEditSchool = new QLineEdit( this, "m_lineEditSchool" );
    m_lineEditSchool->setSizePolicy( QSizePolicy::MinimumExpanding, QSizePolicy::Fixed );
    gridLayout->addMultiCellWidget( m_lineEditSchool, 2, 2, 1, 5 );
    label = new QLabel( i18n( "Schoo&l:" ), this, "textLabelSchool" );
    label->setBuddy( m_lineEditSchool );
    gridLayout->addWidget( label, 2, 0 );

    m_lineEditAddress = new QLineEdit( this, "m_lineEditAddress" );
    m_lineEditAddress->setSizePolicy( QSizePolicy::MinimumExpanding, QSizePolicy::Fixed );
    gridLayout->addMultiCellWidget( m_lineEditAddress, 2, 2, 8, 12 );
    label = new QLabel( i18n( "Add&ress:" ), this, "textLabelAddress" );
    label->setBuddy( m_lineEditAddress );
    gridLayout->addWidget( label, 2, 7 );

    m_lineEditJournal = new QLineEdit( this, "m_lineEditJournal" );
    m_lineEditJournal->setSizePolicy( QSizePolicy::MinimumExpanding, QSizePolicy::Fixed );
    gridLayout->addMultiCellWidget( m_lineEditJournal, 3, 3, 1, 5 );
    label = new QLabel( i18n( "&Journal:" ), this, "textLabelJournal" );
    label->setBuddy( m_lineEditJournal );
    gridLayout->addWidget( label, 3, 0 );

    m_lineEditPages = new QLineEdit( this, "m_lineEditPages" );
    m_lineEditPages->setSizePolicy( QSizePolicy::MinimumExpanding, QSizePolicy::Fixed );
    gridLayout->addMultiCellWidget( m_lineEditPages, 3, 3, 8, 12 );
    label = new QLabel( i18n( "Pa&ges:" ), this, "textLabelPages" );
    label->setBuddy( m_lineEditPages );
    gridLayout->addWidget( label, 3, 7 );

    m_lineEditCrossRef = new QLineEdit( this, "m_lineEditCrossRef" );
    m_lineEditCrossRef->setSizePolicy( QSizePolicy::MinimumExpanding, QSizePolicy::Fixed );
    gridLayout->addMultiCellWidget( m_lineEditCrossRef, 4, 4, 1, 5 );
    label = new QLabel( i18n( "Cro&ss Reference:" ), this, "textLabelCrossRef" );
    label->setBuddy( m_lineEditCrossRef );
    gridLayout->addWidget( label, 4, 0 );

    m_lineEditEdition = new QLineEdit( this, "m_lineEditEdition" );
    m_lineEditEdition->setSizePolicy( QSizePolicy::MinimumExpanding, QSizePolicy::Fixed );
    gridLayout->addWidget( m_lineEditEdition, 5, 1 );
    label = new QLabel( i18n( "E&dition:" ), this, "textLabelEdition" );
    label->setBuddy( m_lineEditEdition );
    gridLayout->addWidget( label, 5, 0 );

    m_lineEditVolume = new QLineEdit( this, "m_lineEditVolume" );
    m_lineEditVolume->setSizePolicy( QSizePolicy::MinimumExpanding, QSizePolicy::Fixed );
    gridLayout->addMultiCellWidget( m_lineEditVolume, 5, 5, 4, 9 );
    label = new QLabel( i18n( "&Volume:" ), this, "textLabelVolume" );
    label->setBuddy( m_lineEditVolume );
    gridLayout->addWidget( label, 5, 3 );

    m_lineEditNumber = new QLineEdit( this, "m_lineEditNumber" );
    m_lineEditNumber->setSizePolicy( QSizePolicy::MinimumExpanding, QSizePolicy::Fixed );
    gridLayout->addWidget( m_lineEditNumber, 5, 12 );
    label = new QLabel( i18n( "Num&ber:" ), this, "textLabelNumber" );
    label->setBuddy( m_lineEditNumber );
    gridLayout->addWidget( label, 5, 11 );

    m_lineEditChapter = new QLineEdit( this, "m_lineEditChapter" );
    m_lineEditChapter->setSizePolicy( QSizePolicy::MinimumExpanding, QSizePolicy::Fixed );
    gridLayout->addWidget( m_lineEditChapter, 6, 1 );
    label = new QLabel( i18n( "Chap&ter:" ), this, "textLabelChapter" );
    label->setBuddy( m_lineEditChapter );
    gridLayout->addWidget( label, 6, 0 );

    m_comboBoxMonth = new QComboBox( FALSE, this, "m_comboBoxMonth" );
    m_comboBoxMonth->setSizePolicy( QSizePolicy::MinimumExpanding, QSizePolicy::Fixed );
    gridLayout->addMultiCellWidget( m_comboBoxMonth, 6, 6, 4, 9 );
    label = new QLabel( i18n( "Mont&h:" ), this, "textLabelMonth" );
    label->setBuddy( m_comboBoxMonth );
    gridLayout->addWidget( label, 6, 3 );

    m_spinBoxYear = new QSpinBox( this, "m_spinBoxYear" );
    m_spinBoxYear->setSizePolicy( QSizePolicy::MinimumExpanding, QSizePolicy::Fixed );
    m_spinBoxYear->setMaxValue( 4000 );
    m_spinBoxYear->setMinValue( 0 );
    m_spinBoxYear->setValue( 2000 );
    gridLayout->addWidget( m_spinBoxYear, 6, 12 );
    label = new QLabel( i18n( "&Year:" ), this, "textLabelYear" );
    label->setBuddy( m_spinBoxYear );
    gridLayout->addWidget( label, 6, 11 );

    m_lineEditISBN = new QLineEdit( this, "m_lineEditISBN" );
    m_lineEditISBN->setSizePolicy( QSizePolicy::MinimumExpanding, QSizePolicy::Fixed );
    gridLayout->addMultiCellWidget( m_lineEditISBN, 7, 7, 1, 5 );
    label = new QLabel( i18n( "ISBN:" ), this, "textLabelISBN" );
    label->setBuddy( m_lineEditISBN );
    gridLayout->addWidget( label, 7, 0 );

    m_lineEditISSN = new QLineEdit( this, "m_lineEditISSN" );
    m_lineEditISSN->setSizePolicy( QSizePolicy::MinimumExpanding, QSizePolicy::Fixed );
    gridLayout->addMultiCellWidget( m_lineEditISSN, 7, 7, 8, 12 );
    label = new QLabel( i18n( "ISSN:" ), this, "textLabelISSN" );
    label->setBuddy( m_lineEditISSN );
    gridLayout->addWidget( label, 7, 7 );

    QSpacerItem *spacer = new QSpacerItem( 10, 10, QSizePolicy::Preferred, QSizePolicy::Maximum );
    gridLayout->addItem( spacer, 0, 6 );
    spacer = new QSpacerItem( 10, 10, QSizePolicy::Preferred, QSizePolicy::Maximum );
    gridLayout->addItem( spacer, 5, 2 );
    spacer = new QSpacerItem( 10, 10, QSizePolicy::Preferred, QSizePolicy::Maximum );
    gridLayout->addItem( spacer, 5, 10 );
    spacer = new QSpacerItem( 10, 10, QSizePolicy::Preferred, QSizePolicy::Expanding );
    gridLayout->addItem( spacer, 11, 0 );

    m_spinBoxYear->setMinValue( 999 );
    m_spinBoxYear->setMaxValue( 3000 );
    m_spinBoxYear->setSpecialValueText( i18n( "not set" ) );

    m_comboBoxMonth->insertItem( i18n( "not set" ) );
    for ( int i = 0; i < 12; i++ )
        m_comboBoxMonth->insertItem( MonthsI18N[ i ] );
}

void KBibTeXEntryWidgetPublication::getEntryData( BibTeXEntry * entry )
{
    BibTeXEntryField * field = entry->getField( BibTeXEntryField::ftHowPublished );
    if ( field )
        m_lineEditHowPublished->setText( field->value() );
    field = entry->getField( BibTeXEntryField::ftPublisher );
    if ( field )
        m_lineEditPublisher->setText( field->value() );
    field = entry->getField( BibTeXEntryField::ftOrganization );
    if ( field )
        m_lineEditOrganization->setText( field->value() );
    field = entry->getField( BibTeXEntryField::ftInstitution );
    if ( field )
        m_lineEditInstitution->setText( field->value() );
    field = entry->getField( BibTeXEntryField::ftSchool );
    if ( field )
        m_lineEditSchool->setText( field->value() );
    field = entry->getField( BibTeXEntryField::ftAddress );
    if ( field )
        m_lineEditAddress->setText( field->value() );
    field = entry->getField( BibTeXEntryField::ftJournal );
    if ( field )
        m_lineEditJournal->setText( field->value() );
    field = entry->getField( BibTeXEntryField::ftCrossRef );
    if ( field )
        m_lineEditCrossRef->setText( field->value() );
    field = entry->getField( BibTeXEntryField::ftISBN );
    if ( field )
        m_lineEditISBN->setText( field->value() );
    field = entry->getField( BibTeXEntryField::ftISSN );
    if ( field )
        m_lineEditISSN->setText( field->value() );
    field = entry->getField( BibTeXEntryField::ftPages );
    if ( field )
        m_lineEditPages->setText( field->value() );
    field = entry->getField( BibTeXEntryField::ftNumber );
    if ( field )
        m_lineEditNumber->setText( field->value() );
    field = entry->getField( BibTeXEntryField::ftChapter );
    if ( field )
        m_lineEditChapter->setText( field->value() );
    field = entry->getField( BibTeXEntryField::ftEdition );
    if ( field )
        m_lineEditEdition->setText( field->value() );
    field = entry->getField( BibTeXEntryField::ftVolume );
    if ( field )
        m_lineEditVolume->setText( field->value() );

    field = entry->getField( BibTeXEntryField::ftYear );
    int year = 0;
    bool ok = false;
    if ( field && ( ( year = field->value().toInt( &ok ) ) ) && ok && ( year >= 1000 ) && ( year <= m_spinBoxYear->maxValue() ) )
        m_spinBoxYear->setValue( year );
    else
        m_spinBoxYear->setValue( m_spinBoxYear->minValue() );
    field = entry->getField( BibTeXEntryField::ftMonth );
    if ( field )
        for ( int i = 0; i < 12; i++ )
            if ( field->value().compare( MonthsTriple[ i ] ) == 0 )
            {
                m_comboBoxMonth->setCurrentItem( i + 1 );
                break;
            }

    installEventFilters( this );
}

bool KBibTeXEntryWidgetPublication::setEntryData( BibTeXEntry * entry )
{
    bool result = true;

    result &= setEntryDataText( entry, m_lineEditHowPublished->text(), BibTeXEntryField::ftHowPublished );
    result &= setEntryDataText( entry, m_lineEditPublisher->text(), BibTeXEntryField::ftPublisher );
    result &= setEntryDataText( entry, m_lineEditOrganization->text(), BibTeXEntryField::ftOrganization );
    result &= setEntryDataText( entry, m_lineEditInstitution->text(), BibTeXEntryField::ftInstitution );
    result &= setEntryDataText( entry, m_lineEditSchool->text(), BibTeXEntryField::ftSchool );
    result &= setEntryDataText( entry, m_lineEditAddress->text(), BibTeXEntryField::ftAddress );
    result &= setEntryDataText( entry, m_lineEditJournal->text(), BibTeXEntryField::ftJournal );
    result &= setEntryDataText( entry, m_lineEditCrossRef->text(), BibTeXEntryField::ftCrossRef );
    result &= setEntryDataText( entry, m_lineEditISBN->text(), BibTeXEntryField::ftISBN );
    result &= setEntryDataText( entry, m_lineEditISSN->text(), BibTeXEntryField::ftISSN );
    result &= setEntryDataText( entry, m_lineEditPages->text(), BibTeXEntryField::ftPages );
    result &= setEntryDataText( entry, m_lineEditChapter->text(), BibTeXEntryField::ftChapter );
    result &= setEntryDataText( entry, m_lineEditNumber->text(), BibTeXEntryField::ftNumber );
    result &= setEntryDataText( entry, m_lineEditEdition->text(), BibTeXEntryField::ftEdition );
    result &= setEntryDataText( entry, m_lineEditVolume->text(), BibTeXEntryField::ftVolume );

    result &= setEntryDataSpinBox( entry, m_spinBoxYear, BibTeXEntryField::ftYear );

    if ( m_comboBoxMonth->currentItem() > 0 )
    {
        QString text = QString( MonthsTriple[ m_comboBoxMonth->currentItem() - 1 ] );
        BibTeXEntryField* field = entry->getField( BibTeXEntryField::ftMonth );
        if ( field == NULL )
        {
            field = new BibTeXEntryField( BibTeXEntryField::ftMonth, text );
            entry->addField( field );
        }
        else
            field->setValue( text );
    }
    else
        entry->deleteField( BibTeXEntryField::ftMonth );

    return result;
}

void KBibTeXEntryWidgetPublication::changeEntryType( BibTeXEntry::EntryType entryType, bool enableAll, bool isReadWrite )
{
    setWidgetStatus( m_lineEditHowPublished, BibTeXEntry::getRequireStatus( entryType, BibTeXEntryField::ftHowPublished ) , enableAll, isReadWrite );
    setWidgetStatus( m_lineEditOrganization, BibTeXEntry::getRequireStatus( entryType, BibTeXEntryField::ftOrganization ), enableAll, isReadWrite );
    setWidgetStatus( m_lineEditSchool, BibTeXEntry::getRequireStatus( entryType, BibTeXEntryField::ftSchool ), enableAll, isReadWrite );
    setWidgetStatus( m_lineEditJournal, BibTeXEntry::getRequireStatus( entryType, BibTeXEntryField::ftJournal ), enableAll, isReadWrite );
    setWidgetStatus( m_lineEditCrossRef, BibTeXEntry::getRequireStatus( entryType, BibTeXEntryField::ftCrossRef ), enableAll, isReadWrite );
    setWidgetStatus( m_lineEditPublisher, BibTeXEntry::getRequireStatus( entryType, BibTeXEntryField::ftPublisher ), enableAll, isReadWrite );
    setWidgetStatus( m_lineEditInstitution, BibTeXEntry::getRequireStatus( entryType, BibTeXEntryField::ftInstitution ), enableAll, isReadWrite );
    setWidgetStatus( m_lineEditAddress, BibTeXEntry::getRequireStatus( entryType, BibTeXEntryField::ftAddress ) , enableAll, isReadWrite );
    setWidgetStatus( m_lineEditISBN, BibTeXEntry::getRequireStatus( entryType, BibTeXEntryField::ftISBN ) , enableAll, isReadWrite );
    setWidgetStatus( m_lineEditISSN, BibTeXEntry::getRequireStatus( entryType, BibTeXEntryField::ftISSN ), enableAll, isReadWrite );
    setWidgetStatus( m_lineEditPages, BibTeXEntry::getRequireStatus( entryType, BibTeXEntryField::ftPages ) , enableAll, isReadWrite );
    setWidgetStatus( m_lineEditEdition, BibTeXEntry::getRequireStatus( entryType, BibTeXEntryField::ftEdition ), enableAll, isReadWrite );
    setWidgetStatus( m_lineEditNumber, BibTeXEntry::getRequireStatus( entryType, BibTeXEntryField::ftNumber ) , enableAll, isReadWrite );
    setWidgetStatus( m_lineEditVolume, BibTeXEntry::getRequireStatus( entryType, BibTeXEntryField::ftVolume ), enableAll, isReadWrite );
    setWidgetStatus( m_lineEditChapter, BibTeXEntry::getRequireStatus( entryType, BibTeXEntryField::ftChapter ), enableAll, isReadWrite );
    setWidgetStatus( m_comboBoxMonth, BibTeXEntry::getRequireStatus( entryType, BibTeXEntryField::ftMonth ), enableAll, isReadWrite );
    setWidgetStatus( m_spinBoxYear, BibTeXEntry::getRequireStatus( entryType, BibTeXEntryField::ftYear ), enableAll, isReadWrite );
}

void KBibTeXEntryWidgetPublication::updateWarnings()
{
    addWarning( BibTeXEntryField::ftPages, i18n( "Pages" ), !m_lineEditPages->text().isEmpty(), m_lineEditPages );
    addWarning( BibTeXEntryField::ftChapter, i18n( "Chapter" ), !m_lineEditChapter->text().isEmpty(), m_lineEditChapter );
    addWarning( BibTeXEntryField::ftNumber, i18n( "Number" ), !m_lineEditNumber->text().isEmpty(), m_lineEditNumber );
    addWarning( BibTeXEntryField::ftOrganization, i18n( "Organization" ), !m_lineEditOrganization->text().isEmpty(), m_lineEditOrganization );
    addWarning( BibTeXEntryField::ftSchool, i18n( "School" ), !m_lineEditSchool->text().isEmpty(), m_lineEditSchool );
    addWarning( BibTeXEntryField::ftCrossRef, i18n( "Cross Reference" ), !m_lineEditCrossRef->text().isEmpty(), m_lineEditCrossRef );
    addWarning( BibTeXEntryField::ftPublisher, i18n( "Publisher" ), !m_lineEditPublisher->text().isEmpty(), m_lineEditPublisher );
    addWarning( BibTeXEntryField::ftInstitution, i18n( "Institution" ), !m_lineEditInstitution->text().isEmpty(), m_lineEditInstitution );
    addWarning( BibTeXEntryField::ftAddress, i18n( "Address" ), !m_lineEditAddress->text().isEmpty(), m_lineEditAddress );
    addWarning( BibTeXEntryField::ftHowPublished, i18n( "How Published" ), !m_lineEditHowPublished->text().isEmpty(), m_lineEditHowPublished );
    addWarning( BibTeXEntryField::ftISBN, i18n( "ISBN" ), !m_lineEditISBN->text().isEmpty(), m_lineEditISBN );
    addWarning( BibTeXEntryField::ftISSN, i18n( "ISSN" ), !m_lineEditISSN->text().isEmpty(), m_lineEditISSN );
    addWarning( BibTeXEntryField::ftJournal, i18n( "Journal" ), !m_lineEditJournal->text().isEmpty(), m_lineEditJournal );
    addWarning( BibTeXEntryField::ftEdition, i18n( "Edition" ), !m_lineEditEdition->text().isEmpty(), m_lineEditEdition );
    addWarning( BibTeXEntryField::ftVolume, i18n( "Volume" ), !m_lineEditVolume->text().isEmpty(), m_lineEditVolume);

    addWarning( BibTeXEntryField::ftYear, i18n( "Year" ), m_spinBoxYear->value() > 0, m_spinBoxYear );

    addWarning( BibTeXEntryField::ftMonth, i18n( "Month" ), m_comboBoxMonth->currentItem() > 0, m_comboBoxMonth );
}

// ============================================================================================

KBibTeXEntryWidgetMisc::KBibTeXEntryWidgetMisc( KBibTeXEntryWidget * entryWidget, QWidget * parent, const char * name )
        : KBibTeXEntryWidgetTab( entryWidget, parent, name )
{
    setupGUI();
}

KBibTeXEntryWidgetMisc::~KBibTeXEntryWidgetMisc()
{
    //TODO
}

void KBibTeXEntryWidgetMisc::setupGUI()
{
    QGridLayout * gridLayout = new QGridLayout( this, 1, 1, KDialog::marginHint(), KDialog::spacingHint(), "gridLayout" );

    m_lineEditType = new QLineEdit( this, "m_lineEditType" );
    gridLayout->addWidget( m_lineEditType, 0, 1 );
    QLabel *label = new QLabel( i18n( "&Type:" ), this, "textLabelType" );
    label->setBuddy( m_lineEditType );
    gridLayout->addWidget( label, 0, 0 );

    label = new QLabel( i18n( "&Key:" ), this, "textLabelKey" );
    gridLayout->addWidget( label, 0, 3 );
    m_lineEditKey = new QLineEdit( this, "m_lineEditKey" );
    label->setBuddy( m_lineEditKey );
    gridLayout->addWidget( m_lineEditKey, 0, 4 );

    m_lineEditNote = new QLineEdit( this, "m_lineEditNote" );
    gridLayout->addMultiCellWidget( m_lineEditNote, 1, 1, 1, 4 );
    label = new QLabel( i18n( "Not&e:" ), this, "textLabelNote" );
    label->setBuddy( m_lineEditNote );
    gridLayout->addWidget( label, 1, 0 );

    QSpacerItem *spacer = new QSpacerItem( 24, 20, QSizePolicy::Fixed, QSizePolicy::Minimum );
    gridLayout->addItem( spacer, 0, 2 );

    m_lineEditAnnote = new QLineEdit( this, "m_lineEditAnnote" );
    gridLayout->addMultiCellWidget( m_lineEditAnnote, 2, 2, 1, 4 );
    label = new QLabel( i18n( "&Annote:" ), this, "textLabelAnnote" );
    label->setBuddy( m_lineEditAnnote );
    gridLayout->addWidget( label, 2, 0 );

    QHBoxLayout* hboxLayout = new QHBoxLayout( 0, 0, 6, "hboxLayout" );
    m_lineEditURL = new QLineEdit( this, "m_lineEditURL" );
    hboxLayout->addWidget( m_lineEditURL );
    label = new QLabel( i18n( "&URL:" ), this, "textLabelURL" );
    label->setBuddy( m_lineEditURL );
    gridLayout->addWidget( label, 3, 0 );
    m_pushButtonURLtoNote = new QPushButton( i18n( "&Apply to field 'Note'" ), this, "m_pushButtonURLtoNote" );
    hboxLayout->addWidget( m_pushButtonURLtoNote );
    m_pushButtonOpenURL = new QPushButton( i18n( "Open" ), this, "m_pushButtonOpenURL" );
    hboxLayout->addWidget( m_pushButtonOpenURL );
    gridLayout->addMultiCellLayout( hboxLayout, 3, 3, 1, 4 );

    m_textEditAbstract = new QTextEdit( this, "m_textEditAbstract" );
    gridLayout->addMultiCellWidget( m_textEditAbstract, 4, 5, 1, 4 );
    label = new QLabel( i18n( "A&bstract:" ), this, "textLabelAbstract" );
    label->setBuddy( m_textEditAbstract );
    gridLayout->addWidget( label, 4, 0 );

    spacer = new QSpacerItem( 20, 160, QSizePolicy::Minimum, QSizePolicy::Expanding );
    gridLayout->addItem( spacer, 5, 0 );

    connect( m_pushButtonOpenURL, SIGNAL( clicked() ), this , SLOT( openURL() ) );
    connect( m_pushButtonURLtoNote, SIGNAL( clicked() ), this, SLOT( applyURLtoNode() ) );

    installEventFilters( this );
}

void KBibTeXEntryWidgetMisc::getEntryData( BibTeXEntry * entry )
{
    BibTeXEntryField * field = entry->getField( BibTeXEntryField::ftType );
    if ( field )
        m_lineEditType->setText( field->value() );
    field = entry->getField( BibTeXEntryField::ftKey );
    if ( field )
        m_lineEditKey->setText( field->value() );
    field = entry->getField( BibTeXEntryField::ftNote );
    if ( field )
        m_lineEditNote->setText( field->value() );
    field = entry->getField( BibTeXEntryField::ftAnnote );
    if ( field )
        m_lineEditAnnote->setText( field->value() );
    field = entry->getField( BibTeXEntryField::ftAbstract );
    if ( field )
        m_textEditAbstract->setText( field->value() );
    field = entry->getField( BibTeXEntryField::ftURL );
    if ( field )
        m_lineEditURL->setText( field->value() );
}

bool KBibTeXEntryWidgetMisc::setEntryData( BibTeXEntry * entry )
{
    bool result = true;

    result &= setEntryDataText( entry, m_lineEditType->text(), BibTeXEntryField::ftType );
    result &= setEntryDataText( entry, m_lineEditKey->text(), BibTeXEntryField::ftKey );
    result &= setEntryDataText( entry, m_lineEditNote->text(), BibTeXEntryField::ftNote );
    result &= setEntryDataText( entry, m_lineEditAnnote->text(), BibTeXEntryField::ftAnnote );
    result &= setEntryDataText( entry, m_textEditAbstract->text(), BibTeXEntryField::ftAbstract );
    result &= setEntryDataText( entry, m_lineEditURL->text(), BibTeXEntryField::ftURL );

    return result;
}

void KBibTeXEntryWidgetMisc::changeEntryType( BibTeXEntry::EntryType entryType, bool enableAll, bool isReadWrite )
{
    setWidgetStatus( m_lineEditType, BibTeXEntry::getRequireStatus( entryType, BibTeXEntryField::ftType ), enableAll, isReadWrite );
    setWidgetStatus( m_lineEditKey, BibTeXEntry::getRequireStatus( entryType, BibTeXEntryField::ftKey ) , enableAll, isReadWrite );
    setWidgetStatus( m_lineEditNote, BibTeXEntry::getRequireStatus( entryType, BibTeXEntryField::ftNote ), enableAll, isReadWrite );
    setWidgetStatus( m_lineEditAnnote, BibTeXEntry::getRequireStatus( entryType, BibTeXEntryField::ftAnnote ), enableAll, isReadWrite );
    setWidgetStatus( m_lineEditURL, BibTeXEntry::getRequireStatus( entryType, BibTeXEntryField::ftURL ), enableAll, isReadWrite );

    m_lineEditURL->setReadOnly( !isReadWrite );
    m_pushButtonOpenURL->setEnabled( isReadWrite );
    m_pushButtonURLtoNote->setEnabled( isReadWrite );
    m_textEditAbstract->setReadOnly( !isReadWrite );
}

void KBibTeXEntryWidgetMisc::updateWarnings()
{
    addWarning( BibTeXEntryField::ftKey, i18n( "Key" ), !m_lineEditKey->text().isEmpty(), m_lineEditKey );
    addWarning( BibTeXEntryField::ftNote, i18n( "Note" ), !m_lineEditNote->text().isEmpty(), m_lineEditNote );
    addWarning( BibTeXEntryField::ftType, i18n( "Type" ), !m_lineEditType->text().isEmpty(), m_lineEditType );
    addWarning( BibTeXEntryField::ftAnnote, i18n( "Annote" ), !m_lineEditAnnote->text().isEmpty(), m_lineEditAnnote );
    addWarning( BibTeXEntryField::ftURL, i18n( "URL" ), !m_lineEditURL->text().isEmpty(), m_lineEditURL );
    addWarning( BibTeXEntryField::ftAbstract, i18n( "Abstract" ), !m_textEditAbstract->text().isEmpty(), m_textEditAbstract );
}

void KBibTeXEntryWidgetMisc::applyURLtoNode()
{
    m_lineEditNote->setText( "\\url{" + m_lineEditURL->text() + "}" );
}

void KBibTeXEntryWidgetMisc::openURL()
{
    kapp->invokeBrowser( m_lineEditURL->text() );
}

// ============================================================================================

KBibTeXEntryWidgetUserFields::KBibTeXEntryWidgetUserFields( KBibTeXEntryWidget * entryWidget, QWidget * parent, const char * name )
        : KBibTeXEntryWidgetTab( entryWidget, parent, name ), m_deletedUserFields( new QStringList() )
{
    setupGUI();
}

KBibTeXEntryWidgetUserFields::~KBibTeXEntryWidgetUserFields()
{
    delete m_deletedUserFields;
}

void KBibTeXEntryWidgetUserFields::setupGUI()
{
    QGridLayout * gridLayout = new QGridLayout( this, 5, 3, KDialog::marginHint(), KDialog::spacingHint(), "gridLayout" );

    m_lineEditUserKey = new QLineEdit( this, "m_lineEditUserKey" );
    gridLayout->addWidget( m_lineEditUserKey, 0, 1 );
    QLabel *label = new QLabel( i18n( "&Key:" ), this );
    label->setBuddy( m_lineEditUserKey );
    gridLayout->addWidget( label, 0, 0 );

    m_pushButtonUserAdd = new QPushButton( i18n( "&Add" ), this, "m_pushButtonUserAdd" );
    gridLayout->addWidget( m_pushButtonUserAdd, 0, 2 );

    m_textEditUserValue = new QTextEdit( this, "m_textEditUserValue" );
    gridLayout->addMultiCellWidget( m_textEditUserValue, 1, 2, 1, 1 );
    label = new QLabel( i18n( "&Value:" ), this );
    label->setBuddy( m_textEditUserValue );
    gridLayout->addWidget( label, 1, 0 );

    QSpacerItem* spacer = new QSpacerItem( 20, 110, QSizePolicy::Minimum, QSizePolicy::Expanding );
    gridLayout->addItem( spacer, 2, 0 );

    m_listViewUserFields = new QListView( this, "m_listViewUserFields" );
    m_listViewUserFields->addColumn( i18n( "Key" ) );
    m_listViewUserFields->addColumn( i18n( "Value" ) );
    m_listViewUserFields->setAllColumnsShowFocus( true );
    //     gridLayout->addWidget( m_listViewUserFields, 4, 1 );
    gridLayout->addMultiCellWidget( m_listViewUserFields, 3, 5, 1, 1 );

    label = new QLabel( i18n( "&List:" ), this );
    label->setBuddy( m_listViewUserFields );
    label->setAlignment( Qt::AlignTop );
    gridLayout->addWidget( label, 3, 0 );

    m_pushButtonUserDelete = new QPushButton( i18n( "&Delete" ), this, "m_pushButtonUserDelete" );
    gridLayout->addWidget( m_pushButtonUserDelete, 3, 2 );

    m_pushButtonUserOpen = new QPushButton( i18n( "Op&en" ), this, "m_pushButtonUserOpen" );
    gridLayout->addWidget( m_pushButtonUserOpen, 4, 2 );

    spacer = new QSpacerItem( 20, 110, QSizePolicy::Minimum, QSizePolicy::Expanding );
    gridLayout->addItem( spacer, 5, 2 );

    m_pushButtonUserOpen->setEnabled( FALSE );
    m_pushButtonUserAdd->setEnabled( FALSE );
    m_pushButtonUserDelete->setEnabled( FALSE );

    connect( m_listViewUserFields, SIGNAL( clicked( QListViewItem*, const QPoint&, int ) ), this, SLOT( userFieldExecute( QListViewItem* ) ) );
    connect( m_lineEditUserKey, SIGNAL( textChanged( const QString& ) ), this, SLOT( userNameChanged( const QString& ) ) );
    connect( m_pushButtonUserAdd, SIGNAL( clicked( ) ), this, SLOT( userAddClicked( ) ) );
    connect( m_pushButtonUserDelete, SIGNAL( clicked( ) ), this, SLOT( userDeleteClicked( ) ) );
    connect( m_pushButtonUserOpen, SIGNAL( clicked() ), this, SLOT( userOpenClicked() ) );

    installEventFilters( this );
}

void KBibTeXEntryWidgetUserFields::getEntryData( BibTeXEntry * entry )
{
    for ( QListViewItemIterator it( m_listViewUserFields ); it.current(); it++ )
        delete it.current();
    m_listViewUserFields->clear();

    for ( QValueList<BibTeXEntryField*>::iterator it = entry->begin(); it != entry->end(); it++ )
    {
        if ( ( *it ) ->fieldType() == BibTeXEntryField::ftUnknown )
        {
            BibTeXEntryField * field = new BibTeXEntryField( *it );
            QListView *parent = m_listViewUserFields;
            new KBibTeXFieldListViewItem( field, parent, "KBibTeXFieldListViewItem" );
        }
    }
}

bool KBibTeXEntryWidgetUserFields::setEntryData( BibTeXEntry * entry )
{
    for ( QStringList::iterator it = m_deletedUserFields->begin(); it != m_deletedUserFields->end(); it++ )
        entry->deleteField( *it );
    for ( QListViewItemIterator it( m_listViewUserFields ); it.current(); it++ )
    {
        entry->deleteField( it.current() ->text( 0 ) );
        BibTeXEntryField* field = new BibTeXEntryField( it.current() ->text( 0 ), it.current() ->text( 1 ) );
        entry->addField( field );
    }

    return true;
}

void KBibTeXEntryWidgetUserFields::changeEntryType( BibTeXEntry::EntryType /* entryType */, bool /* enableAll */, bool isReadWrite )
{
    setWidgetStatus( m_lineEditUserKey, BibTeXEntryField::frsOptional, TRUE, isReadWrite );
    setWidgetStatus( m_pushButtonUserAdd, BibTeXEntryField::frsOptional, TRUE, isReadWrite );
    setWidgetStatus( m_pushButtonUserDelete, BibTeXEntryField::frsOptional, TRUE, isReadWrite );
    setWidgetStatus( m_textEditUserValue, BibTeXEntryField::frsOptional, TRUE, isReadWrite );
}

void KBibTeXEntryWidgetUserFields::updateWarnings()
{
    //TODO
}

void KBibTeXEntryWidgetUserFields::userFieldExecute( QListViewItem * item )
{
    KBibTeXFieldListViewItem * flvi = dynamic_cast<KBibTeXFieldListViewItem*>( item );
    if ( flvi != NULL && flvi->field() != NULL )
    {
        BibTeXEntryField * field = flvi->field();
        m_lineEditUserKey->setText( field->key() );
        m_textEditUserValue->setText( field->value() );

        KURL url( field->value() );
        m_pushButtonUserOpen->setEnabled( url.isValid() );
    }
    else
    {
        qDebug( "flvi == NULL || flvi->field() == NULL : This should never happen!" );
        m_pushButtonUserOpen->setEnabled( FALSE );
    }
}

void KBibTeXEntryWidgetUserFields::userNameChanged( const QString & newText )
{
    QListViewItem * item = m_listViewUserFields->findItem( newText, 0 ) ;
    bool contains = item != NULL;
    bool isUnknown = BibTeXEntryField::fieldTypeFromString( newText ) == BibTeXEntryField::ftUnknown;

    if ( contains )
        m_listViewUserFields->setSelected( item, TRUE );
    m_pushButtonUserDelete->setEnabled( contains );
    m_pushButtonUserAdd->setEnabled( !newText.isEmpty() && isUnknown );
    m_pushButtonUserAdd->setText( contains ? i18n( "&Apply" ) : i18n( "&Add" ) );
}

void KBibTeXEntryWidgetUserFields::userAddClicked( )
{
    KBibTeXFieldListViewItem * flvi = dynamic_cast<KBibTeXFieldListViewItem*>( m_listViewUserFields->findItem( m_lineEditUserKey->text(), 0 ) );

    if ( flvi != NULL )
    {
        // Apply
        BibTeXEntryField * field = flvi->field();
        field->setValue( m_textEditUserValue->text() );
        flvi->updateItem();
    }
    else
    {
        // Add
        BibTeXEntryField * field = new BibTeXEntryField( m_lineEditUserKey->text(), m_textEditUserValue->text() );
        m_listViewUserFields->setSelected( new KBibTeXFieldListViewItem( field, m_listViewUserFields ), true );
    }

    m_pushButtonUserAdd->setText( i18n( "&Apply" ) );
    m_pushButtonUserDelete->setEnabled( TRUE );

    KURL url( m_textEditUserValue->text() );
    m_pushButtonUserOpen->setEnabled( url.isValid() );
}

void KBibTeXEntryWidgetUserFields::userDeleteClicked()
{
    QListViewItem * item = m_listViewUserFields->findItem( m_lineEditUserKey->text(), 0 );

    if ( item != NULL )
    {
        m_deletedUserFields->append( item->text( 0 ) );
        m_listViewUserFields->removeItem( item );
        m_pushButtonUserAdd->setText( i18n( "&Add" ) );
        m_pushButtonUserDelete->setEnabled( FALSE );
        m_pushButtonUserOpen->setEnabled( FALSE );
    }
}

void KBibTeXEntryWidgetUserFields::userOpenClicked()
{
    kapp->invokeBrowser( m_textEditUserValue->text() );
}

// ============================================================================================

KBibTeXEntryWidgetSource::KBibTeXEntryWidgetSource( KBibTeXEntryWidget * entryWidget, QWidget * parent, const char * name )
        : KBibTeXEntryWidgetTab( entryWidget, parent, name )
{
    setupGUI();
}

KBibTeXEntryWidgetSource::~KBibTeXEntryWidgetSource()
{
    //TODO
}

void KBibTeXEntryWidgetSource::setupGUI()
{
    QHBoxLayout * hboxLayout = new QHBoxLayout( this, KDialog::marginHint(), KDialog::spacingHint(), "boxLayout" );

    m_textEditSource = new QTextEdit( this );
    m_textEditSource->setFont( KGlobalSettings::fixedFont() );
    hboxLayout->addWidget( m_textEditSource );

    installEventFilters( this );
}

void KBibTeXEntryWidgetSource::getEntryData( BibTeXEntry * entry )
{
    QString display = entry->toString( BibTeXFile::formatBibTeX, BibTeXFile::encUTF8 );
    m_textEditSource->setText( display );
}

bool KBibTeXEntryWidgetSource::setEntryData( BibTeXEntry * entry )
{
    BibTeXFile * bibtexfile = new BibTeXFile();
    QString text( m_textEditSource->text() );
    bibtexfile->insertText( text );

    bool result = bibtexfile->count() == 1 ;
    if ( result )
    {
        BibTeXElement * element = bibtexfile->at( 0 );
        BibTeXEntry* parsedEntry = dynamic_cast<BibTeXEntry*>( element );
        result = parsedEntry != NULL ;
        if ( result )
        {
            entry->setEntryType( parsedEntry->entryType() );
            entry->setId( parsedEntry->id() );
            for ( QValueList<BibTeXEntryField*>::iterator it = parsedEntry->begin(); it != parsedEntry->end(); it++ )
                entry->addField( new BibTeXEntryField( *it ) );
        }
    }

    delete bibtexfile;

    return result;
}
void KBibTeXEntryWidgetSource::changeEntryType( BibTeXEntry::EntryType /* entryType */, bool /* enableAll */, bool isReadWrite )
{
    setWidgetStatus( m_textEditSource, BibTeXEntryField::frsOptional, TRUE, isReadWrite );
}

void KBibTeXEntryWidgetSource::updateWarnings()
{
    //TODO
}

bool KBibTeXEntryWidgetSource::isModified()
{
    return m_textEditSource->isModified();
}

void KBibTeXEntryWidgetSource::setModified( bool modified )
{
    m_textEditSource->setModified( modified );
}

#include "kbibtexentrywidgettab.moc"
