/***************************************************************************
*   Copyright (C) 2004-2005 by Thomas Fischer                             *
*   fischer@unix-ag.uni-kl.de                                             *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
#include <qlabel.h>
#include <qmultilineedit.h>
#include <qlayout.h>
#include <kdialogbase.h>
#include <klocale.h>

#include <bibtexpreamble.h>

#include "kbibtexpreamblewidget.h"

KBibTeXPreambleWidget::KBibTeXPreambleWidget( BibTeXPreamble *preamble, bool isReadWrite, QWidget *parent, const char *name )
        : QWidget( parent, name ), m_preamble( preamble )
{
    setupGUI();
    getPreambleData();
    m_multiLineEditPreamble->setReadOnly( !isReadWrite );
}


KBibTeXPreambleWidget::~KBibTeXPreambleWidget()
{}

void KBibTeXPreambleWidget::setupGUI()
{
    QVBoxLayout * layout = new QVBoxLayout( this, 0, KDialog::spacingHint() );
    QLabel *label = new QLabel( i18n( "&Preamble:" ), this );
    layout->addWidget( label );
    label->show();
    m_multiLineEditPreamble = new QMultiLineEdit( this );
    layout->addWidget( m_multiLineEditPreamble );
    m_multiLineEditPreamble->show();
    label->setBuddy( m_multiLineEditPreamble );

}

void KBibTeXPreambleWidget::setPreambleData()
{
    m_preamble->setPreamble( m_multiLineEditPreamble->text() );
}

void KBibTeXPreambleWidget::getPreambleData()
{
    m_multiLineEditPreamble->setText( m_preamble->preamble() );
}

QDialog::DialogCode KBibTeXPreambleWidget::execute( BibTeXPreamble *preamble, const QString& caption, bool isReadWrite, QWidget *parent, const char *name )
{
    KDialogBase * dlg = new KDialogBase( parent, name, true, caption, KDialogBase::Ok | KDialogBase::Cancel );
    KBibTeXPreambleWidget* ui = new KBibTeXPreambleWidget( preamble, isReadWrite, dlg, "kbibtexpreamblewidget" );

    dlg->setMainWidget( ui );
    QDialog::DialogCode result = ( QDialog::DialogCode ) dlg->exec();
    if ( isReadWrite && result == QDialog::Accepted )
        ui->setPreambleData();

    delete( ui );
    delete( dlg );

    return result;
}

#include "kbibtexpreamblewidget.moc"
