/***************************************************************************
*   Copyright (C) 2004-2005 by Thomas Fischer                             *
*   fischer@unix-ag.uni-kl.de                                             *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
#include <qlabel.h>
#include <qmultilineedit.h>
#include <qlineedit.h>
#include <qlayout.h>
#include <kdialogbase.h>
#include <klocale.h>

#include <bibtexstring.h>

#include "kbibtexstringwidget.h"

KBibTeXStringWidget::KBibTeXStringWidget( BibTeXString *bibtexstring, bool isReadWrite, QWidget *parent, const char *name )
        : QWidget( parent, name ), m_bibtexstring( bibtexstring )
{
    setupGUI();
    getStringData();
    m_lineEditStringId->setReadOnly( !isReadWrite );
    m_multiLineEditStringValue->setReadOnly( !isReadWrite );
}

KBibTeXStringWidget::~KBibTeXStringWidget()
{
    // TODO
}

void KBibTeXStringWidget::setupGUI()
{
    setMinimumWidth( 384 );
    QVBoxLayout * layout = new QVBoxLayout( this, 0, KDialog::spacingHint() );

    QLabel *label = new QLabel( i18n( "String &id:" ), this );
    layout->addWidget( label );
    label->show();
    m_lineEditStringId = new QLineEdit( this );
    layout->addWidget( m_lineEditStringId );
    m_lineEditStringId->show();
    label->setBuddy( m_lineEditStringId );

    label = new QLabel( i18n( "String &value:" ), this );
    layout->addWidget( label );
    label->show();
    m_multiLineEditStringValue = new QMultiLineEdit( this );
    layout->addWidget( m_multiLineEditStringValue );
    m_multiLineEditStringValue->show();
    label->setBuddy( m_multiLineEditStringValue );
}

void KBibTeXStringWidget::setStringData()
{
    QString text = m_lineEditStringId->text();
    m_bibtexstring->setKey( text );
    text = m_multiLineEditStringValue->text();
    m_bibtexstring->setValue( text );
}

void KBibTeXStringWidget::getStringData()
{
    m_lineEditStringId->setText( m_bibtexstring->key() );
    m_multiLineEditStringValue->setText( m_bibtexstring->value() );
}


QDialog::DialogCode KBibTeXStringWidget::execute( BibTeXString *bibtexstring, const QString& caption, bool isReadWrite, QWidget *parent, const char *name )
{
    KDialogBase * dlg = new KDialogBase( parent, name, true, caption, KDialogBase::Ok | KDialogBase::Cancel );
    KBibTeXStringWidget* ui = new KBibTeXStringWidget( bibtexstring, isReadWrite, dlg, "kbibtexstringwidget" );

    dlg->setMainWidget( ui );
    QDialog::DialogCode result = ( QDialog::DialogCode ) dlg->exec();
    if ( isReadWrite && result == QDialog::Accepted )
        ui->setStringData();

    delete( ui );
    delete( dlg );

    return result;
}

#include "kbibtexstringwidget.moc"
